; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Nom du projet : Language System - Module
; Nom du fichier : Language System - Module.pb
; Version du fichier : 1.1.0
; Programmation : OK
; Programm par : Guillaume Saumure
; Alias : StarBootics
; Courriel : gsaumure@cgocable.ca
; Date : 13-02-2015
; Mise  jour : 13-02-2015
; Code PureBasic : V5.31
; Plateforme : Windows, Linux, MacOS X
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

DeclareModule Language
  
  Declare.s GetFileName()
  Declare Initialize(P_PathName.s, P_FileName.s, PtrToGUI.i = #Null)
  Declare Reset()
  Declare LoadTableFile()
  Declare WritePrefs(P_GroupName.s)
  Declare ReadPrefs(P_GroupName.s)
  Declare.s Message(P_Message.s)
  Declare Requester()
  Declare To_GUI()
  
  CompilerIf #PB_Compiler_Debugger
    
    ; These procedures only exist to create default language files
    ; and they are created only when the main program is compiled
    ; with the debugger active.
    
    Declare CreateTableFile()
    Declare AddTableElement(P_Key.s, P_Message.s)
    
  CompilerEndIf 
  
EndDeclareModule

Module Language
  
  Structure Instance 
    
    PathName.s
    FileName.s
    RemoteToGUI.i
    Map Table.s()
    
  EndStructure 

  Global Instance.Instance 
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Procdures prives <<<<<
  
  Procedure GetLanguageXMLNode(CurrentNode)
    
    If ParentXMLNode(CurrentNode) = #Null
      StructNode = ChildXMLNode(CurrentNode)
      If GetXMLNodeName(StructNode) = "Language"
        Success = #True
      EndIf
    Else
      Success = #True
      StructNode = CurrentNode
    EndIf
    
    If Success = #True
      
      FieldNode = ChildXMLNode(StructNode)
      
      While FieldNode
        
        Select GetXMLNodeName(FieldNode)
            
          Case "Table"
            Table_Max = Val(GetXMLAttribute(FieldNode, "ElementCount"))
            
            For Element = 0 To Table_Max - 1
              IndexNode = ChildXMLNode(FieldNode, Element + 1)
              If GetXMLNodeName(IndexNode) = "Element"
                AddMapElement(Instance\Table(), GetXMLAttribute(IndexNode, "Key"))
                Instance\Table() = GetXMLAttribute(IndexNode, "Table")
              EndIf
            Next
            
        EndSelect
        
        FieldNode = NextXMLNode(FieldNode)
        
      Wend
      
    EndIf
    
  EndProcedure 
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Nom de la procdure <<<<<
  
  Procedure.s GetFileName()
  
     ProcedureReturn Instance\FileName
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Procdures publiques <<<<<
  
  Procedure Initialize(P_PathName.s, P_FileName.s, PtrToGUI.i = #Null)
    
    Instance\PathName = P_PathName
    Instance\FileName = P_FileName
    Instance\RemoteToGUI = PtrToGUI
    
  EndProcedure
  
  Procedure Reset()
    
    Instance\PathName = ""
    Instance\FileName = ""
    Instance\RemoteToGUI = #Null
    ClearMap(Instance\Table()) 
    
  EndProcedure
  
  Procedure LoadTableFile()
    
    FileID = LoadXML(#PB_Any, Instance\PathName + Instance\FileName)
    
    If IsXML(FileID)
      FormatXML(FileID, #PB_XML_CutNewline)
      GetLanguageXMLNode(RootXMLNode(FileID))
      FreeXML(FileID)
    EndIf
    
  EndProcedure
  
  Procedure WritePrefs(P_GroupName.s)
    
    PreferenceGroup(P_GroupName)
    WritePreferenceString("FileName", Instance\FileName)
      
  EndProcedure
  
  Procedure ReadPrefs(P_GroupName.s)
    
    PreferenceGroup(P_GroupName)
    Instance\FileName = ReadPreferenceString("FileName", Instance\FileName)
    
  EndProcedure

  Procedure.s Message(P_Message.s)
    
    If FindMapElement(Instance\Table(), P_Message)
      Message_Out.s = Instance\Table(P_Message)
    Else
      Message_Out = P_Message
    EndIf
    
    ProcedureReturn Message_Out
  EndProcedure
  
  Procedure To_GUI()
    
    If Instance\RemoteToGUI <> #Null
      CallFunctionFast(Instance\RemoteToGUI)
    EndIf
    
  EndProcedure
  
  Procedure Requester()
    
    LanguageRquester_Handle = OpenWindow(#PB_Any, 0, 0, 220, 130, Message("Language Requester"), #PB_Window_ScreenCentered)
    
    If LanguageRquester_Handle <> 0
      
      LanguageRquester_List_Handle = ListViewGadget(#PB_Any, 10, 10, 200, 75)
      LanguageRquester_Btn_OK_Handle = ButtonGadget(#PB_Any, 10, 90, 95, 35, Message("OK"))
      LanguageRquester_Btn_Cancel_Handle = ButtonGadget(#PB_Any, 115, 90, 95, 35, Message("Cancel"))
      
      StickyWindow(LanguageRquester_Handle, 1)  
      
      ; On met en place la liste des langues disponibles
      
      If ExamineDirectory(0, Instance\PathName, "*.xml") <> 0
        
        While NextDirectoryEntry(0) <> 0
          
          langues.s = DirectoryEntryName(0)
          
          If Left(langues.s, 1) <> "."
            
            AddGadgetItem(LanguageRquester_List_Handle, -1, StringField(GetFilePart(langues), 1, "."))
            
          EndIf
          
        Wend
        
      EndIf
      
      Repeat
        
        EventID = WaitWindowEvent()
        
        If EventID = #PB_Event_Gadget
          
          Select EventGadget() ; La gestion des venements
              
            Case LanguageRquester_Btn_OK_Handle
              
              Language.s = GetGadgetText(LanguageRquester_List_Handle) + ".xml"
              
              If Instance\FileName <> Language
                
                Instance\FileName = Language
                LoadTableFile()
                
                If Instance\RemoteToGUI <> #Null
                  CallFunctionFast(Instance\RemoteToGUI)
                EndIf

              EndIf
              
              EventID = #PB_Event_CloseWindow
              
            Case LanguageRquester_Btn_Cancel_Handle
              EventID = #PB_Event_CloseWindow
              
          EndSelect
          
        EndIf
        
      Until EventID = #PB_Event_CloseWindow
      
      CloseWindow(LanguageRquester_Handle)
      
    EndIf
    
  EndProcedure
  
  CompilerIf #PB_Compiler_Debugger
    
    ; These procedures only exist to create default language files
    ; and they are created only when the main program is compiled
    ; with the debugger active.
    
    Procedure SetLanguageXMLNode(CurrentNode)
      
      If ParentXMLNode(CurrentNode) = #Null
        StructNode = CreateXMLNode(CurrentNode, "Language")
      Else
        StructNode = CurrentNode
      EndIf
      
      FieldNode = CreateXMLNode(StructNode, "Table")
      SetXMLAttribute(FieldNode, "ElementCount", Str(MapSize(Instance\Table())))
      Element = 0
      
      ForEach Instance\Table()
        IndexNode = CreateXMLNode(FieldNode, "Element")
        SetXMLAttribute(IndexNode, "Key", MapKey(Instance\Table()))
        SetXMLAttribute(IndexNode, "Table", Instance\Table())
      Next
      
    EndProcedure
    
    Procedure CreateTableFile()
      
      FileID = CreateXML(#PB_Any)
      
      If IsXML(FileID)
        SetLanguageXMLNode(RootXMLNode(FileID))
        FormatXML(FileID, #PB_XML_ReFormat | #PB_XML_ReIndent)
        SaveXML(FileID, Instance\PathName + Instance\FileName)
        FreeXML(FileID)
      EndIf
      
    EndProcedure
    
    Procedure AddTableElement(P_Key.s, P_Message.s)
      
      Instance\Table(P_Key) = P_Message
      
    EndProcedure
  
  CompilerEndIf 
  
EndModule

; <<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< FIN DU FICHIER <<<<<
; <<<<<<<<<<<<<<<<<<<<<<<<<<
; IDE Options = PureBasic 5.70 beta 1 (Linux - x64)
; Folding = ----
; EnableXP
; CompileSourceDirectory